@@ -30,8 +30,7 @@
 #include <glib-object.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
-
-#include <gconf/gconf-client.h>
+#include <cairo-xlib.h>
 
 #include "gsm-inhibit-dialog.h"
 #include "gsm-store.h"
@@ -46,8 +45,6 @@
 
 #define GSM_INHIBIT_DIALOG_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GSM_TYPE_INHIBIT_DIALOG, GsmInhibitDialogPrivate))
 
-#define IS_STRING_EMPTY(x) ((x)==NULL||(x)[0]=='\0')
-
 #define GTKBUILDER_FILE "gsm-inhibit-dialog.ui"
 
 #ifndef DEFAULT_ICON_SIZE
@@ -316,80 +313,44 @@ scale_pixbuf (GdkPixbuf *pixbuf,
 
 #ifdef HAVE_XRENDER
 
-/* adapted from metacity */
-static GdkColormap*
-get_cmap (GdkPixmap *pixmap)
-{
-        GdkColormap *cmap;
-
-        cmap = gdk_drawable_get_colormap (pixmap);
-        if (cmap) {
-                g_object_ref (G_OBJECT (cmap));
-        }
-
-        if (cmap == NULL) {
-                if (gdk_drawable_get_depth (pixmap) == 1) {
-                        g_debug ("Using NULL colormap for snapshotting bitmap\n");
-                        cmap = NULL;
-                } else {
-                        g_debug ("Using system cmap to snapshot pixmap\n");
-                        cmap = gdk_screen_get_system_colormap (gdk_drawable_get_screen (pixmap));
-
-                        g_object_ref (G_OBJECT (cmap));
-                }
-        }
-
-        /* Be sure we aren't going to blow up due to visual mismatch */
-        if (cmap &&
-            (gdk_colormap_get_visual (cmap)->depth !=
-             gdk_drawable_get_depth (pixmap))) {
-                cmap = NULL;
-                g_debug ("Switching back to NULL cmap because of depth mismatch\n");
-        }
-
-        return cmap;
-}
-
 static GdkPixbuf *
-pixbuf_get_from_pixmap (Pixmap xpixmap)
+pixbuf_get_from_pixmap (Display *display,
+                        Pixmap   xpixmap,
+                        int      width,
+                        int      height)
 {
-        GdkDrawable *drawable;
-        GdkPixbuf   *retval;
-        GdkColormap *cmap;
-        int          width;
-        int          height;
+        cairo_surface_t *surface;
+        GdkPixbuf       *retval;
+        Visual          *visual;
 
         retval = NULL;
-        cmap = NULL;
 
+        visual = DefaultVisual (display, 0);
         g_debug ("GsmInhibitDialog: getting foreign pixmap for %u", (guint)xpixmap);
-        drawable = gdk_pixmap_foreign_new (xpixmap);
-        if (GDK_IS_PIXMAP (drawable)) {
-                cmap = get_cmap (drawable);
-                gdk_drawable_get_size (drawable,
-                                       &width,
-                                       &height);
+        surface = cairo_xlib_surface_create (display,
+                                             xpixmap,
+                                             visual,
+                                             width,
+                                             height);
+        if (surface != NULL) {
                 g_debug ("GsmInhibitDialog: getting pixbuf w=%d h=%d", width, height);
+                retval = gdk_pixbuf_get_from_surface (surface,
+                                                      0, 0,
+                                                      width, height);
+        }
 
-                retval = gdk_pixbuf_get_from_drawable (NULL,
-                                                       drawable,
-                                                       cmap,
-                                                       0, 0,
-                                                       0, 0,
-                                                       width, height);
-        }
-        if (cmap) {
-                g_object_unref (G_OBJECT (cmap));
-        }
-        if (drawable) {
-                g_object_unref (G_OBJECT (drawable));
+        if (surface) {
+                cairo_surface_destroy (surface);
         }
 
         return retval;
 }
 
 static Pixmap
-get_pixmap_for_window (Window window)
+get_pixmap_for_window (Display *display,
+                       Window   window,
+                       int     *widthp,
+                       int     *heightp)
 {
         XWindowAttributes        attr;
         XRenderPictureAttributes pa;
@@ -403,9 +364,9 @@ get_pixmap_for_window (Window window)
         int                      width;
         int                      height;
 
-        XGetWindowAttributes (GDK_DISPLAY (), window, &attr);
+        XGetWindowAttributes (display, window, &attr);
 
-        format = XRenderFindVisualFormat (GDK_DISPLAY (), attr.visual);
+        format = XRenderFindVisualFormat (display, attr.visual);
         has_alpha = (format->type == PictTypeDirect && format->direct.alphaMask);
         x = attr.x;
         y = attr.y;
@@ -414,15 +375,15 @@ get_pixmap_for_window (Window window)
 
         pa.subwindow_mode = IncludeInferiors; /* Don't clip child widgets */
 
-        src_picture = XRenderCreatePicture (GDK_DISPLAY (), window, format, CPSubwindowMode, &pa);
+        src_picture = XRenderCreatePicture (display, window, format, CPSubwindowMode, &pa);
 
-        pixmap = XCreatePixmap (GDK_DISPLAY (),
+        pixmap = XCreatePixmap (display,
                                 window,
                                 width, height,
                                 attr.depth);
 
-        dst_picture = XRenderCreatePicture (GDK_DISPLAY (), pixmap, format, 0, 0);
-        XRenderComposite (GDK_DISPLAY (),
+        dst_picture = XRenderCreatePicture (display, pixmap, format, 0, 0);
+        XRenderComposite (display,
                           has_alpha ? PictOpOver : PictOpSrc,
                           src_picture,
                           None,
@@ -431,6 +392,12 @@ get_pixmap_for_window (Window window)
                           0, 0,
                           width, height);
 
+        if (widthp != NULL) {
+                *widthp = width;
+        }
+        if (heightp != NULL) {
+                *heightp = height;
+        }
 
         return pixmap;
 }
@@ -438,17 +405,24 @@ get_pixmap_for_window (Window window)
 #endif /* HAVE_COMPOSITE */
 
 static GdkPixbuf *
-get_pixbuf_for_window (guint xid,
-                       int   width,
-                       int   height)
+get_pixbuf_for_window (GdkDisplay *gdkdisplay,
+                       guint       xid,
+                       int         thumb_width,
+                       int         thumb_height)
 {
         GdkPixbuf *pixbuf = NULL;
 #ifdef HAVE_XRENDER
+        Display   *display;
         Window     xwindow;
         Pixmap     xpixmap;
+        int        width;
+        int        height;
 
+        display = GDK_DISPLAY_XDISPLAY (gdkdisplay);
         xwindow = (Window) xid;
-        xpixmap = get_pixmap_for_window (xwindow);
+
+        xpixmap = get_pixmap_for_window (display, xwindow, &width, &height);
+
         if (xpixmap == None) {
                 g_debug ("GsmInhibitDialog: Unable to get window snapshot for %u", xid);
                 return NULL;
@@ -456,19 +430,18 @@ get_pixbuf_for_window (guint xid,
                 g_debug ("GsmInhibitDialog: Got xpixmap %u", (guint)xpixmap);
         }
 
-        pixbuf = pixbuf_get_from_pixmap (xpixmap);
+        pixbuf = pixbuf_get_from_pixmap (display, xpixmap, width, height);
 
         if (xpixmap != None) {
-                gdk_error_trap_push ();
-                XFreePixmap (GDK_DISPLAY (), xpixmap);
-                gdk_display_sync (gdk_display_get_default ());
-                gdk_error_trap_pop ();
+                gdk_x11_display_error_trap_push (gdkdisplay);
+                XFreePixmap (display, xpixmap);
+                gdk_x11_display_error_trap_pop_ignored (gdkdisplay);
         }
 
         if (pixbuf != NULL) {
                 GdkPixbuf *scaled;
                 g_debug ("GsmInhibitDialog: scaling pixbuf to w=%d h=%d", width, height);
-                scaled = scale_pixbuf (pixbuf, width, height, TRUE);
+                scaled = scale_pixbuf (pixbuf, thumb_width, thumb_height, TRUE);
                 g_object_unref (pixbuf);
                 pixbuf = scaled;
         }
@@ -482,6 +455,7 @@ static void
 add_inhibitor (GsmInhibitDialog *dialog,
                GsmInhibitor     *inhibitor)
 {
+        GdkDisplay     *gdkdisplay;
         const char     *name;
         const char     *icon_name;
         const char     *app_id;
@@ -493,6 +467,8 @@ add_inhibitor (GsmInhibitDialog *dialog,
         guint           xid;
         char           *freeme;
 
+        gdkdisplay = gtk_widget_get_display (GTK_WIDGET (dialog));
+
         /* FIXME: get info from xid */
 
         desktop_file = NULL;
@@ -513,7 +489,7 @@ add_inhibitor (GsmInhibitDialog *dialog,
         xid = gsm_inhibitor_peek_toplevel_xid (inhibitor);
         g_debug ("GsmInhibitDialog: inhibitor has XID %u", xid);
         if (xid > 0 && dialog->priv->have_xrender) {
-                pixbuf = get_pixbuf_for_window (xid, DEFAULT_SNAPSHOT_SIZE, DEFAULT_SNAPSHOT_SIZE);
+                pixbuf = get_pixbuf_for_window (gdkdisplay, xid, DEFAULT_SNAPSHOT_SIZE, DEFAULT_SNAPSHOT_SIZE);
                 if (pixbuf == NULL) {
                         g_debug ("GsmInhibitDialog: unable to read pixbuf from %u", xid);
                 }
@@ -664,7 +640,7 @@ update_dialog_text (GsmInhibitDialog *dialog)
         if (model_has_one_entry (GTK_TREE_MODEL (dialog->priv->list_store))) {
                 g_debug ("Found one entry in model");
                 header_text = _("A program is still running:");
-                description_text = _("Waiting for program to finish.  Interrupting program may cause you to lose work.");
+                description_text = _("Waiting for the program to finish.  Interrupting the program may cause you to lose work.");
         } else {
                 g_debug ("Found multiple entries (or none) in model");
                 header_text = _("Some programs are still running:");
@@ -901,7 +877,7 @@ setup_dialog (GsmInhibitDialog *dialog)
                 button_text = _("Switch User Anyway");
                 break;
         case GSM_LOGOUT_ACTION_LOGOUT:
-                button_text = _("Logout Anyway");
+                button_text = _("Log Out Anyway");
                 break;
         case GSM_LOGOUT_ACTION_SLEEP:
                 button_text = _("Suspend Anyway");
@@ -910,7 +886,7 @@ setup_dialog (GsmInhibitDialog *dialog)
                 button_text = _("Hibernate Anyway");
                 break;
         case GSM_LOGOUT_ACTION_SHUTDOWN:
-                button_text = _("Shutdown Anyway");
+                button_text = _("Shut Down Anyway");
                 break;
         case GSM_LOGOUT_ACTION_REBOOT:
                 button_text = _("Reboot Anyway");
@@ -982,22 +958,27 @@ gsm_inhibit_dialog_constructor (GType                  type,
                                 GObjectConstructParam *construct_properties)
 {
         GsmInhibitDialog *dialog;
+#ifdef HAVE_XRENDER
+        GdkDisplay *gdkdisplay;
+#endif /* HAVE_XRENDER */
 
         dialog = GSM_INHIBIT_DIALOG (G_OBJECT_CLASS (gsm_inhibit_dialog_parent_class)->constructor (type,
                                                                                                                   n_construct_properties,
                                                                                                                   construct_properties));
 
 #ifdef HAVE_XRENDER
-        gdk_error_trap_push ();
-        if (XRenderQueryExtension (GDK_DISPLAY (), &dialog->priv->xrender_event_base, &dialog->priv->xrender_error_base)) {
+        gdkdisplay = gdk_display_get_default ();
+
+        gdk_x11_display_error_trap_push (gdkdisplay);
+        if (XRenderQueryExtension (GDK_DISPLAY_XDISPLAY (gdkdisplay), &dialog->priv->xrender_event_base, &dialog->priv->xrender_error_base)) {
                 g_debug ("GsmInhibitDialog: Initialized XRender extension");
                 dialog->priv->have_xrender = TRUE;
         } else {
                 g_debug ("GsmInhibitDialog: Unable to initialize XRender extension");
                 dialog->priv->have_xrender = FALSE;
         }
-        gdk_display_sync (gdk_display_get_default ());
-        gdk_error_trap_pop ();
+        gdk_display_sync (gdkdisplay);
+        gdk_x11_display_error_trap_pop_ignored (gdkdisplay);
 #endif /* HAVE_XRENDER */
 
         /* FIXME: turn this on when it is ready */
@@ -1116,12 +1097,10 @@ gsm_inhibit_dialog_init (GsmInhibitDialog *dialog)
         gtk_container_add (GTK_CONTAINER (content_area), widget);
 
         gtk_container_set_border_width (GTK_CONTAINER (dialog), 6);
-        gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
         gtk_window_set_icon_name (GTK_WINDOW (dialog), "system-log-out");
         gtk_window_set_title (GTK_WINDOW (dialog), "");
         g_object_set (dialog,
-                      "allow-shrink", FALSE,
-                      "allow-grow", FALSE,
+                      "resizable", FALSE,
                       NULL);
 }
 
